// animations
var animationPause = 20; // milliseconds

var currentTimerId = null;
var currentAnimatedElements = {};
var currentVisibleElements = {};

function calcSpeed(speedDir, maxheight) {
    var speed = Math.floor(maxheight / 12) + 1;
    return speedDir >= 0 ? speed : -speed;
}

function addAnimation(name, speed, dontRecordVisible) {
    if ( name in currentAnimatedElements ) {
        currentAnimatedElements[name].speed = calcSpeed(speed, currentAnimatedElements[name].maxheight);
    }
    else {
        var el = document.getElementById(name);
        if ( el == null )
            return;

        el.style.display = 'block';
        var height = el.offsetHeight; // the current height

        el.style.height = 'auto';
        var maxheight = el.offsetHeight; // the maximal height

        currentAnimatedElements[name] = {
            speed :calcSpeed(speed, maxheight),
            height :height,
            maxheight :maxheight
        };
        el.style.height = height + 'px';

        if ( !dontRecordVisible )
            currentVisibleElements[name] = true;
    }
    // possibly start timer
    if ( currentTimerId == null )
        currentTimerId = setInterval('updateAnimations()', animationPause);
}

function updateAnimations() {
    var newAnims = {};
    var count = 0;

    for ( var prop in currentAnimatedElements ) {
        var data = currentAnimatedElements[prop];

        var el = document.getElementById(prop);
        if ( el == null )
            continue;

        if ( data.height + data.speed <= 0 ) {
            el.style.display = 'none';
            el.style.height = '0px';
            delete currentVisibleElements[prop];
        }
        else if ( data.height + data.speed >= data.maxheight ) {
            el.style.height = "auto";
        }
        else {
            data.height += data.speed;
            el.style.height = data.height + "px";
            newAnims[prop] = data;
            count++;
        }
    }
    currentAnimatedElements = newAnims;

    if ( count == 0 ) { // all animations finished
        clearInterval(currentTimerId);
        currentTimerId = null;
    }
}

function clearAnimations() {
    if ( currentTimerId != null ) {
        clearInterval(currentTimerId);
        currentTimerId = null;
    }
    for ( var prop in currentVisibleElements ) {
        var el = document.getElementById(prop);
        if ( el != null ) {
            el.style.display = 'none';
            el.style.height = '0px';
        }
    }
    currentVisibleElements = {};
    currentAnimatedElements = {};
}

function toggleOpenEntry(id) {
    var img = document.getElementById(id + "_img");
    if ( img == null )
        return;

    if ( img.className == "unfold_open" ) {
        img.className = "unfold_close";
        addAnimation(id + "_entry", -20, true);
    }
    else {
        img.className = "unfold_open";
        addAnimation(id + "_entry", 20, true);
    }
}


function getPageParameters() {
    var query = window.location.search.substring(1);
    var parms = query.split('&');
    var result = {};
    for ( var i = 0; i < parms.length; i++ ) {
        pv = parms[i].split('=');
        if ( pv.length == 2 )
            result[pv[0]] = pv[1];
    }
    return result;
}

function crossXHR() {
    var crossxhr = false;

    if ( window.XMLHttpRequest ) {
        crossxhr = new XMLHttpRequest();
        if ( crossxhr.overrideMimeType )
            crossxhr.overrideMimeType('text/xml');
    }
    else if ( window.ActiveXObject ) {
        try {
            crossxhr = new ActiveXObject('Msxml2.XMLHTTP');
        } catch ( e ) {
            try {
                crossxhr = new ActiveXObject('Microsoft.XMLHTTP');
            } catch ( e ) {
                crossxhr = false;
            }
        }
    }
    return crossxhr;
}


function isInCollection(el, col) {
    for ( var i = 0; i < col.length; i++ )
        if ( col[i] == el )
            return true;
    return false;
}

function allParents(node) {
    var parents = [];
    for ( var p = node; p != null; p = p.parentNode )
        parents[parents.length] = p;
    return parents;
}

function filterMask(elements, idMask, maxnum) {
    var res = [];
    for ( var i = 0; i < elements.length && (maxnum == null || res.length < maxnum); i++ )
        if ( elements[i].id != null && elements[i].id.match(idMask) )
            res[res.length] = elements[i];
    return res;
}

function forEach(arr, func) {
    for ( var i in arr )
        func(arr[i]);
}

function forEachWhile(arr, func) {
    for ( var i in arr )
        if ( !func(arr[i]) )
            break;
}


function colorBranch(el) {
    if ( el == null )
        return;
    // clear others
    var allLinks = document.getElementById('theOneAndOnlySidebar').getElementsByTagName('a');
    for ( var l in allLinks )
        if ( allLinks[l].className == 'onbranch' )
            allLinks[l].className = '';
    // mark branch
    var parents = allParents(el);
    for ( var i in parents ) {
        if ( parents[i].id != null && parents[i].id.match(/u?h2_\d+_li/) ) {
            var link = parents[i].getElementsByTagName('a')[0];
            link.className = 'onbranch';
        }
    }
}

function openBranchSlow(li) {
    if ( li == null )
        return;

    var lisup = filterMask(allParents(li), /^(uh2_\d+)_ul$/);

    var speed = 10;

    for ( var i = 1; document.getElementById("uh2_" + i + "_li") != null; i++ ) {
        var el = document.getElementById("uh2_" + i + "_ul");
        if ( el != null && !isInCollection(el, lisup) )
            addAnimation(el.id, -speed, true);
    }

    for ( var j in lisup )
        addAnimation(lisup[j].id, speed, true);
}

function openBranchInstantly(li) {
    if ( li == null )
        return;

    for ( var el = li; el != null && el.id != null; el = el.parentNode ) {
        var m = el.id.match(/(uh2_\d+)_ul/);
        if ( m ) {
            el.style.display = "block";
            el.style.height = "auto";
        }
    }
}

function findFirstLI(kategorija) {
    if ( kategorija.match(/^u/) ) {
        // an UL id
        var ul = document.getElementById(kategorija + "_ul");
        if ( ul == null )
            return null;
        return filterMask(ul.getElementsByTagName("li"), /^(h2_\d+)_li$/, 1)[0];
    }
    else {
        // already a LI id
        return document.getElementById(kategorija + '_li');
    }
}

function categoryFromLI(li) {
    if ( li ) {
        var match = li.id.match(/^(h2_\d+)_li$/);
        if ( match )
            return match[1];
    }
    return null;
}

function showCategoryPage(kategorija) {
    clearAnimations();
    // close others
    for ( var i = 1; ; i++ ) {
        var el = document.getElementById("h2_" + i + "_page");
        if ( el == null )
            break;
        el.style.display = "none";
    }
    // show this
    if ( kategorija ) {
        var thisel = document.getElementById(kategorija + "_page");
        if ( thisel != null )
            thisel.style.display = "block";
    }
}

var currentCategory = null;

function showCategory(kategorija, slow) {
    currentCategory = kategorija;

    var li = findFirstLI(kategorija);
    showCategoryPage(categoryFromLI(li));
    if ( slow )
        openBranchSlow(li);
    else
        openBranchInstantly(li);
    colorBranch(li);
}

function prikaziInit(kategorija) {
    showCategory(kategorija, false);
}

function prikazi(kategorija) {
    showCategory(kategorija, true);
}


function getPageBookmark() {
    var id = window.location.hash.substring(1);
    if ( id == undefined || id == '' )
        id = 'h2_1';
    return id;
}

function checkSectionChange(oldid) {
    var id = getPageBookmark();
    if ( id != oldid )
        prikazi(id);
    setTimeout('checkSectionChange("' + id + '")', 100);
}

function showInitial() {
    var id = getPageBookmark();
    prikaziInit(id);
    // add timer for location checking
    setTimeout('checkSectionChange("' + id + '")', 100);
}
